Norm_Lapl_Transform <- function(Distr_data_1, Distr_data_2, bin_width_log, threshold_log_ref, fitnum, find_by_ranking = TRUE){
   
  Distr_data_2 <- rowSums(Distr_data_2);
  Distr_data_log_INI <- log(Distr_data_1);
  Distr_data_log_REF <- log(Distr_data_2);
  
  hist_INI <- Create_hist_overlapped(Distr_data_log_INI, bin_width_log)$PDF
  hist_REF <- Create_hist_overlapped(Distr_data_log_REF, bin_width_log)$PDF
  
  PDF_untr_INI_X = c(hist_INI$mids[1] - bin_width_log / 2, hist_INI$mids, hist_INI$mids[length(hist_INI$mids)] + bin_width_log / 2);
  PDF_untr_INI_Y = c(hist_INI$density[1], hist_INI$density, hist_INI$density[length(hist_INI$density)]);
  
  PDF_untr_REF_X = c(hist_REF$mids[1] - bin_width_log / 2, hist_REF$mids, hist_REF$mids[length(hist_REF$mids)] + bin_width_log / 2);
  PDF_untr_REF_Y = c(hist_REF$density[1], hist_REF$density, hist_REF$density[length(hist_REF$density)]);
  
  require(Bolstad2) ## uses Simpson's rule
  CDF_INI <- sintegral(PDF_untr_INI_X, PDF_untr_INI_Y,  n.pts = 2048);
  PDF_INI_full <- list(x=PDF_untr_INI_X, y=PDF_untr_INI_Y);
  PDF_REF_full <- list(x=PDF_untr_REF_X, y=PDF_untr_REF_Y);
  
  CDF_REF <- sintegral(PDF_REF_full$x, PDF_REF_full$y,  n.pts = 2048);
  
  CDF_INI$y <- CDF_INI$y + (1 - max(CDF_INI$y, na.rm=TRUE));
  CDF_REF$y <- CDF_REF$y + (1 - max(CDF_REF$y, na.rm=TRUE)); ## Reference curve to which we normalize
  CDF_INI_full <- CDF_INI;

  
  CDF_REF_full <- list(x=CDF_REF$x, y=CDF_REF$y);
  #Tresholding procedure
  tmp_stop_val <- 0;
  for (j in 1:length(CDF_REF$x)){
    if(CDF_REF$x[j] < threshold_log_ref){
      tmp_stop_val <- tmp_stop_val + 1;    
    }   
  }   
  
  #Number of genes cut off in reference distribution
  N_genes_cutoff <- length(which(Distr_data_log_REF < threshold_log_ref))
  sorted_INI_genes <- sort(log(Distr_data_1+1), decreasing = FALSE, na.last = NA, index.return = TRUE)$ix  
  treshold_Ini_Min_Val_ByRanking <- min(Distr_data_log_INI[sorted_INI_genes[N_genes_cutoff:length(sorted_INI_genes)]])
     
  CDF_treshold_Y <- CDF_REF$y[tmp_stop_val]; # The area under PDF we cut 
  CDF_REF_trunc <- list(x = CDF_REF$x[tmp_stop_val : length(CDF_REF$x)], y = CDF_REF$y[tmp_stop_val : length(CDF_REF$x)]);
  
  
  tmp_stop_val <- 1;
  if(find_by_ranking == TRUE){        
    treshold_Ini_Min_Val <- treshold_Ini_Min_Val_ByRanking    
    tmp_stop_val <- max(which(CDF_INI$x <= treshold_Ini_Min_Val))   
  }
  else{    
    tmp_stop_val <- max(which(CDF_INI$y < CDF_treshold_Y))        
    treshold_Ini_Min_Val <- CDF_INI$x[tmp_stop_val];  
  }
  
  treshold_Ref_Min_Val <- threshold_log_ref;
  CDF_INI_trunc <- list(x = CDF_INI$x[tmp_stop_val : length(CDF_INI$y)], y = CDF_INI$y[tmp_stop_val : length(CDF_INI$y)]);
  
  # Remove reads in the distributions below found thresholds
  hist_INI_Tr <- Truncate_distr(PDF_INI_full, treshold_Ini_Min_Val);
  hist_REF_Tr <- Truncate_distr(PDF_REF_full, treshold_Ref_Min_Val);
  
  #Fitting
  x <- hist_INI_Tr$mids;
  res_fit_INI <- nls(hist_INI_Tr$density ~ My_dnl(x, mu, sigma, alpha, beta), lower = list(mu = 4.0, sigma = 0.6, alpha = 1.1, beta = 0.25), upper = list(mu = 9.0, sigma = 1.6, alpha = 1.8, beta = 0.48), start = list(mu = 4, sigma = 0.8, alpha = 1.7, beta = 0.3), algorithm = "port");
  x <- hist_REF_Tr$mids;
  res_fit_REF <- nls(hist_REF_Tr$density ~ My_dnl(x, mu, sigma, alpha, beta), lower = list(mu = 3.0, sigma = 0.6, alpha = 1.5, beta = 0.15), upper = list(mu = 10, sigma = 2.4, alpha = 2.5, beta = 0.38), start = list(mu = 6.0, sigma = 1.2, alpha = 1.7, beta = 0.3), algorithm = "port");
  
  param_fit_REF <- list(mu = summary(res_fit_REF)$parameters[1,1], sigma = summary(res_fit_REF)$parameters[2,1], alpha = summary(res_fit_REF)$parameters[3,1], beta = summary(res_fit_REF)$parameters[4,1]);
  param_fit_INI <- list(mu = summary(res_fit_INI)$parameters[1,1], sigma = summary(res_fit_INI)$parameters[2,1], alpha = summary(res_fit_INI)$parameters[3,1], beta = summary(res_fit_INI)$parameters[4,1]);
  
  x_pdf_fitted_REF <- seq(treshold_Ini_Min_Val - bin_width_log*2, max(hist_REF_Tr$mids) + bin_width_log*2, by = 0.001);
  y_pdf_fitted_REF <- My_dnl(x_pdf_fitted_REF, param_fit_REF$mu, param_fit_REF$sigma, param_fit_REF$alpha, param_fit_REF$beta);
  
  x_pdf_fitted_INI <- seq(treshold_Ini_Min_Val - bin_width_log*2, max(hist_INI_Tr$mids) + bin_width_log*2, by = 0.001);
  y_pdf_fitted_INI <- My_dnl(x_pdf_fitted_INI, param_fit_INI$mu, param_fit_INI$sigma, param_fit_INI$alpha, param_fit_INI$beta);
  
  require(Bolstad2) 
  CDF_INI <- sintegral(x_pdf_fitted_INI, y_pdf_fitted_INI, n.pts = 4096);
  CDF_REF <- sintegral(x_pdf_fitted_REF, y_pdf_fitted_REF, n.pts = 4096);
  
  CDF_INI$y <- CDF_INI$y + (1 - CDF_INI$int);
  CDF_REF$y <- CDF_REF$y + (1 - CDF_REF$int); ## Reference curve to which we normalize
    
  Norm_coeff <- list(x = rep(c(NA), length(Distr_data_1)), Val = rep(c(NA), length(Distr_data_1)));
  Norm_coeff_tr <- list(x = rep(c(NA), length(Distr_data_1)), Val = rep(c(NA), length(Distr_data_1)));
  
  for (j in 1: length(Distr_data_1)){    
    X_Ini <- Distr_data_log_INI[j];
    
    # Check if X is going to be normalized 
    flag_coeff_found <- 0;
    if(!is.na(X_Ini) && !is.infinite(X_Ini)){
      if(X_Ini > treshold_Ini_Min_Val){        
        ## Find corresponding Y-coord initial
        coeff_1 <- max(which(CDF_INI$x <= X_Ini))
        if(!is.na(coeff_1) && coeff_1 >0 && coeff_1 < length(CDF_INI$x)){
          Y_Ini <- CDF_INI$y[coeff_1]
          flag_coeff_found <- flag_coeff_found + 1;
        }      
        
        ## Find corresponding X-coord in the reference CDF
        coeff_2 <- max(which(CDF_REF$y <= Y_Ini))
        if(!is.na(coeff_2) && coeff_2 > 0 && coeff_2 < length(CDF_REF$y)){
          X_Ref <- CDF_REF$x[coeff_2+1]
          flag_coeff_found <- flag_coeff_found + 1;          
        }
        
        if(flag_coeff_found == 2){          
          Norm_coeff$x[j] <- X_Ini
          #browser()
          Norm_coeff$Val[j] <- X_Ref/X_Ini
          Norm_coeff_tr$x[j]<- X_Ini
          Norm_coeff_tr$Val[j] <- X_Ref/X_Ini          
        }else{
          browser()
        }
      }
    }    
  } 

  #The normalization procedure itself
  Data_set_normal = rep(c(NA), length( Norm_coeff$Val ));
  Output_data <- Distr_data_1; #Not norm initial data
  Not_norm_vec <- Distr_data_1;
  k <- 1;
  for (i in 1: length( Norm_coeff$Val )){
    Coeff_norm  <- Norm_coeff$Val[i];
    flag_norm <- 0;    
    if(!is.na(Norm_coeff$Val[i])) { 
      Data_set_normal[k] <- log(Not_norm_vec[i]) * Coeff_norm;
      Output_data[i] <- exp((log(Output_data[i]) * Coeff_norm));     
      k <- k+1;
    }    
  }
  
  cutoff_norm <- treshold_Ref_Min_Val+bin_width_log*0.5
  hist_INI_Norm<- Create_hist_overlapped(log(Output_data), bin_width_log)$PDF 
  hist_INI_Norm <- Truncate_distr(list(x=hist_INI_Norm$mids, y=hist_INI_Norm$density), cutoff_norm);
  
  PDF_INI_Norm <- list(x=hist_INI_Norm$mids, y=hist_INI_Norm$density);
  CDF_INI_norm <- sintegral(PDF_INI_Norm$x, PDF_INI_Norm$y);
  CDF_INI_norm$y <- CDF_INI_norm$y + (1 - CDF_INI_norm$int);
  
  
  Distr_data_out_log <- log(Output_data);
  for (j in 1:length(Distr_data_out_log))
    if(is.infinite(Distr_data_out_log [j]))
      Distr_data_out_log[j] = NA;
    
  PDF_REF_tr <- list(x=hist_REF_Tr$mids, y=hist_REF_Tr$density);
  
  sorted_norm_coeff_x <- sort(Norm_coeff_tr$x, decreasing = FALSE, index.return = TRUE)
  Norm_coeff_tr$x <- sorted_norm_coeff_x$x;
  Norm_coeff_tr$Val <- Norm_coeff_tr$Val[sorted_norm_coeff_x$ix];
  norm_coeff_table <- list(x=Norm_coeff_tr$x, y=Norm_coeff_tr$Val);
  
  
  PDF_REF_tr_x <- treshold_Ref_Min_Val
  PDF_INI_trunc_x <- treshold_Ini_Min_Val;
  CDF_REF_trunc_y <- min(CDF_REF_trunc$y);
  
  PDF_REF_fit <- list(x = x_pdf_fitted_REF, y = y_pdf_fitted_REF); 
  Param_ref_fit <- param_fit_REF; 
  Names_genes_below_threshold <- intersect(sorted_INI_genes[1:N_genes_cutoff], which(Distr_data_log_INI >= 0))

  output_data_list=list(Param_ref_fit=Param_ref_fit, Names_genes_below_threshold=Names_genes_below_threshold, table=Output_data, PDF_REF_fit=PDF_REF_fit, PDF_REF_tr_x = PDF_REF_tr_x, PDF_REF_tr=PDF_REF_tr, PDF_INI_trunc_x = PDF_INI_trunc_x, CDF_REF_trunc_y=CDF_REF_trunc_y, PDF_INI_full=PDF_INI_full, PDF_REF_full=PDF_REF_full, CDF_INI_full=CDF_INI_full, CDF_REF_full=CDF_REF_full, PDF_INI_Norm=PDF_INI_Norm, CDF_INI_norm=CDF_INI_norm, norm_coeff_table=norm_coeff_table);
  
  return(output_data_list)
}








